package com.nsgk.agentcentersdk.api;

import cn.hutool.core.util.URLUtil;
import com.nsgk.agentcentersdk.core.NSProtocol;
import com.nsgk.agentcentersdk.core.NSReportObject;
import com.nsgk.agentcentersdk.entity.NSEntityBase;
import com.nsgk.agentcentersdk.err.NSAssert;
import com.nsgk.agentcentersdk.err.NSErrGlobal;
import com.nsgk.agentcentersdk.err.NSErrno;
import com.nsgk.agentcentersdk.network.NSNetworkManager;
import com.nsgk.agentcentersdk.network.NSNetworkRequest;
import com.nsgk.agentcentersdk.network.NSNetworkResponse;
import com.nsgk.agentcentersdk.utility.NSCrypto;
import com.nsgk.agentcentersdk.utility.NSStr;

import java.util.Objects;

// SDK客户端
/*
请求:
    <host>:<port>/<api>?identifier=<identifier>&protocol=<protocol>&timestamp=<timestamp>
        header: sign
        body: 加密json
响应:
    body:
        code: 200
        msg: ""
        timestamp:
        data: "" // 加密json
 */
public final class NSSDKClient
{
    private final String host;
    private final short port;
    private final String identifier;
    private final String publicKey;

    NSSDKClient(String host, short port, String identifier, String publicKey)
    {
        this.host = host;
        this.port = port;
        this.identifier = identifier;
        this.publicKey = publicKey;
    }

    // 发送请求
    public <T extends NSEntityBase> NSApiResult Send(int protocol, T object)
    {
        String url;
        NSReportObject<T> reportObject;
        NSNetworkRequest request;

        NSAssert.NotNull(object, NSErrno.ERRNO_CLI_DATA_MISSING);
        NSAssert.NotEmpty(identifier, NSErrno.ERRNO_CLI_PARAMETER_MISSING);
        NSErrGlobal.ASSERT(NSProtocol.IsValid(protocol), NSErrno.ERRNO_SYS_INVALID_PROTOCOL);
        url = NSApi.NS_API_REPORT;
        reportObject = new NSReportObject<>(identifier, protocol, object);
        request = new NSNetworkRequest();
        request.setUrl(BuildUrl(url));
        WriteDataObject(request, reportObject);
        NSNetworkResponse response = NSNetworkManager.Post(request);
        NSErrGlobal.ASSERT(response.IsSuccess(), NSErrno.ERRNO_SYS_HTTP);
        String json = response.getData();
        return NSApiResult.FromJSON(json);
    }

    private String BuildUrl(String url)
    {
        return URLUtil.completeUrl(host + ":" + port, url);
    }

    private <T> void WriteDataObject(NSNetworkRequest request, NSReportObject<T> object)
    {
        String dataStr;

        request.ClearHeaders().AddHeader("sign", object.getSign());
        request.ClearQueries().AddQuery("identifier", object.getIdentifier())
                      .AddQuery("protocol", object.getProtocol())
                      .AddQuery("timestamp", object.getTimestamp())
        ;
        dataStr = object.getDataStr();
        try
        {
            dataStr = NSCrypto.RSAEncrypt(dataStr, publicKey);
            NSAssert.NotEmpty(dataStr, NSErrno.ERRNO_SYS_ENCRYPT_FAIL);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            NSErrGlobal.Throw(NSErrno.ERRNO_SYS_ENCRYPT_FAIL);
            return;
        }
        request.setData(dataStr);
    }
}
